from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *


class MAT_SequenceOfArc:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Length(self) -> int: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class MAT_SequenceOfBasicElt:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Length(self) -> int: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class MAT_Side(IntEnum):
	MAT_Left: int = ...
	MAT_Right: int = ...
MAT_Left = MAT_Side.MAT_Left
MAT_Right = MAT_Side.MAT_Right

class MAT_Arc(Standard_Transient):
	def __init__(self, ArcIndex: int, GeomIndex: int, FirstElement: MAT_BasicElt, SecondElement: MAT_BasicElt) -> None: ...
	def FirstElement(self) -> MAT_BasicElt: ...
	def FirstNode(self) -> MAT_Node: ...
	def GeomIndex(self) -> int: ...
	def HasNeighbour(self, aNode: MAT_Node, aSide: MAT_Side) -> bool: ...
	def Index(self) -> int: ...
	def Neighbour(self, aNode: MAT_Node, aSide: MAT_Side) -> MAT_Arc: ...
	def SecondElement(self) -> MAT_BasicElt: ...
	def SecondNode(self) -> MAT_Node: ...
	def SetFirstArc(self, aSide: MAT_Side, anArc: MAT_Arc) -> None: ...
	def SetFirstElement(self, aBasicElt: MAT_BasicElt) -> None: ...
	def SetFirstNode(self, aNode: MAT_Node) -> None: ...
	def SetGeomIndex(self, anInteger: int) -> None: ...
	def SetIndex(self, anInteger: int) -> None: ...
	def SetNeighbour(self, aSide: MAT_Side, aNode: MAT_Node, anArc: MAT_Arc) -> None: ...
	def SetSecondArc(self, aSide: MAT_Side, anArc: MAT_Arc) -> None: ...
	def SetSecondElement(self, aBasicElt: MAT_BasicElt) -> None: ...
	def SetSecondNode(self, aNode: MAT_Node) -> None: ...
	def TheOtherNode(self, aNode: MAT_Node) -> MAT_Node: ...

class MAT_BasicElt(Standard_Transient):
	def __init__(self, anInteger: int) -> None: ...
	def EndArc(self) -> MAT_Arc: ...
	def GeomIndex(self) -> int: ...
	def Index(self) -> int: ...
	def SetEndArc(self, anArc: MAT_Arc) -> None: ...
	def SetGeomIndex(self, anInteger: int) -> None: ...
	def SetIndex(self, anInteger: int) -> None: ...
	def SetStartArc(self, anArc: MAT_Arc) -> None: ...
	def StartArc(self) -> MAT_Arc: ...

class MAT_Bisector(Standard_Transient):
	def __init__(self) -> None: ...
	def AddBisector(self, abisector: MAT_Bisector) -> None: ...
	@overload
	def BisectorNumber(self, anumber: int) -> None: ...
	@overload
	def BisectorNumber(self) -> int: ...
	@overload
	def DistIssuePoint(self, areal: float) -> None: ...
	@overload
	def DistIssuePoint(self) -> float: ...
	def Dump(self, ashift: int, alevel: int) -> None: ...
	@overload
	def EndPoint(self, apoint: int) -> None: ...
	@overload
	def EndPoint(self) -> int: ...
	def FirstBisector(self) -> MAT_Bisector: ...
	@overload
	def FirstEdge(self, anedge: MAT_Edge) -> None: ...
	@overload
	def FirstEdge(self) -> MAT_Edge: ...
	@overload
	def FirstParameter(self, aparameter: float) -> None: ...
	@overload
	def FirstParameter(self) -> float: ...
	@overload
	def FirstVector(self, avector: int) -> None: ...
	@overload
	def FirstVector(self) -> int: ...
	@overload
	def IndexNumber(self, anumber: int) -> None: ...
	@overload
	def IndexNumber(self) -> int: ...
	@overload
	def IssuePoint(self, apoint: int) -> None: ...
	@overload
	def IssuePoint(self) -> int: ...
	def LastBisector(self) -> MAT_Bisector: ...
	def List(self) -> MAT_ListOfBisector: ...
	@overload
	def SecondEdge(self, anedge: MAT_Edge) -> None: ...
	@overload
	def SecondEdge(self) -> MAT_Edge: ...
	@overload
	def SecondParameter(self, aparameter: float) -> None: ...
	@overload
	def SecondParameter(self) -> float: ...
	@overload
	def SecondVector(self, avector: int) -> None: ...
	@overload
	def SecondVector(self) -> int: ...
	@overload
	def Sense(self, asense: float) -> None: ...
	@overload
	def Sense(self) -> float: ...

class MAT_Edge(Standard_Transient):
	def __init__(self) -> None: ...
	@overload
	def Distance(self, adistance: float) -> None: ...
	@overload
	def Distance(self) -> float: ...
	def Dump(self, ashift: int, alevel: int) -> None: ...
	@overload
	def EdgeNumber(self, anumber: int) -> None: ...
	@overload
	def EdgeNumber(self) -> int: ...
	@overload
	def FirstBisector(self, abisector: MAT_Bisector) -> None: ...
	@overload
	def FirstBisector(self) -> MAT_Bisector: ...
	@overload
	def IntersectionPoint(self, apoint: int) -> None: ...
	@overload
	def IntersectionPoint(self) -> int: ...
	@overload
	def SecondBisector(self, abisector: MAT_Bisector) -> None: ...
	@overload
	def SecondBisector(self) -> MAT_Bisector: ...

class MAT_Graph(Standard_Transient):
	def __init__(self) -> None: ...
	def Arc(self, Index: int) -> MAT_Arc: ...
	def BasicElt(self, Index: int) -> MAT_BasicElt: ...
	def ChangeBasicElt(self, Index: int) -> MAT_BasicElt: ...
	def ChangeBasicElts(self, NewMap: MAT_DataMapOfIntegerBasicElt) -> None: ...
	def CompactArcs(self) -> None: ...
	def CompactNodes(self) -> None: ...
	def FusionOfBasicElts(self, IndexElt1: int, IndexElt2: int) -> Tuple[bool, int, int, bool, int, int]: ...
	def Node(self, Index: int) -> MAT_Node: ...
	def NumberOfArcs(self) -> int: ...
	def NumberOfBasicElts(self) -> int: ...
	def NumberOfInfiniteNodes(self) -> int: ...
	def NumberOfNodes(self) -> int: ...
	def Perform(self, SemiInfinite: bool, TheRoots: MAT_ListOfBisector, NbBasicElts: int, NbArcs: int) -> None: ...

class MAT_ListOfBisector(Standard_Transient):
	def __init__(self) -> None: ...
	def BackAdd(self, anitem: MAT_Bisector) -> None: ...
	def Brackets(self, anindex: int) -> MAT_Bisector: ...
	@overload
	def Current(self) -> MAT_Bisector: ...
	@overload
	def Current(self, anitem: MAT_Bisector) -> None: ...
	def Dump(self, ashift: int, alevel: int) -> None: ...
	def First(self) -> None: ...
	def FirstItem(self) -> MAT_Bisector: ...
	def FrontAdd(self, anitem: MAT_Bisector) -> None: ...
	def Index(self) -> int: ...
	def Init(self, aniten: MAT_Bisector) -> None: ...
	def IsEmpty(self) -> bool: ...
	def Last(self) -> None: ...
	def LastItem(self) -> MAT_Bisector: ...
	def LinkAfter(self, anitem: MAT_Bisector) -> None: ...
	def LinkBefore(self, anitem: MAT_Bisector) -> None: ...
	def Loop(self) -> None: ...
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def NextItem(self) -> MAT_Bisector: ...
	def Number(self) -> int: ...
	def Permute(self) -> None: ...
	def Previous(self) -> None: ...
	def PreviousItem(self) -> MAT_Bisector: ...
	def Unlink(self) -> None: ...

class MAT_ListOfEdge(Standard_Transient):
	def __init__(self) -> None: ...
	def BackAdd(self, anitem: MAT_Edge) -> None: ...
	def Brackets(self, anindex: int) -> MAT_Edge: ...
	@overload
	def Current(self) -> MAT_Edge: ...
	@overload
	def Current(self, anitem: MAT_Edge) -> None: ...
	def Dump(self, ashift: int, alevel: int) -> None: ...
	def First(self) -> None: ...
	def FirstItem(self) -> MAT_Edge: ...
	def FrontAdd(self, anitem: MAT_Edge) -> None: ...
	def Index(self) -> int: ...
	def Init(self, aniten: MAT_Edge) -> None: ...
	def IsEmpty(self) -> bool: ...
	def Last(self) -> None: ...
	def LastItem(self) -> MAT_Edge: ...
	def LinkAfter(self, anitem: MAT_Edge) -> None: ...
	def LinkBefore(self, anitem: MAT_Edge) -> None: ...
	def Loop(self) -> None: ...
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def NextItem(self) -> MAT_Edge: ...
	def Number(self) -> int: ...
	def Permute(self) -> None: ...
	def Previous(self) -> None: ...
	def PreviousItem(self) -> MAT_Edge: ...
	def Unlink(self) -> None: ...

class MAT_Node(Standard_Transient):
	def __init__(self, GeomIndex: int, LinkedArc: MAT_Arc, Distance: float) -> None: ...
	def Distance(self) -> float: ...
	def GeomIndex(self) -> int: ...
	def Index(self) -> int: ...
	def Infinite(self) -> bool: ...
	def LinkedArcs(self, S: MAT_SequenceOfArc) -> None: ...
	def NearElts(self, S: MAT_SequenceOfBasicElt) -> None: ...
	def OnBasicElt(self) -> bool: ...
	def PendingNode(self) -> bool: ...
	def SetIndex(self, anIndex: int) -> None: ...
	def SetLinkedArc(self, anArc: MAT_Arc) -> None: ...

class MAT_TListNodeOfListOfBisector(Standard_Transient):
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, anitem: MAT_Bisector) -> None: ...
	def Dummy(self) -> None: ...
	def GetItem(self) -> MAT_Bisector: ...
	@overload
	def Next(self) -> MAT_TListNodeOfListOfBisector: ...
	@overload
	def Next(self, atlistnode: MAT_TListNodeOfListOfBisector) -> None: ...
	@overload
	def Previous(self) -> MAT_TListNodeOfListOfBisector: ...
	@overload
	def Previous(self, atlistnode: MAT_TListNodeOfListOfBisector) -> None: ...
	def SetItem(self, anitem: MAT_Bisector) -> None: ...

class MAT_TListNodeOfListOfEdge(Standard_Transient):
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, anitem: MAT_Edge) -> None: ...
	def Dummy(self) -> None: ...
	def GetItem(self) -> MAT_Edge: ...
	@overload
	def Next(self) -> MAT_TListNodeOfListOfEdge: ...
	@overload
	def Next(self, atlistnode: MAT_TListNodeOfListOfEdge) -> None: ...
	@overload
	def Previous(self) -> MAT_TListNodeOfListOfEdge: ...
	@overload
	def Previous(self, atlistnode: MAT_TListNodeOfListOfEdge) -> None: ...
	def SetItem(self, anitem: MAT_Edge) -> None: ...

class MAT_Zone(Standard_Transient):
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, aBasicElt: MAT_BasicElt) -> None: ...
	def ArcOnFrontier(self, Index: int) -> MAT_Arc: ...
	def Limited(self) -> bool: ...
	def NoEmptyZone(self) -> bool: ...
	def NumberOfArcs(self) -> int: ...
	def Perform(self, aBasicElt: MAT_BasicElt) -> None: ...

# harray1 classes
# harray2 classes
# hsequence classes




from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.BinMDF import *
from OCC.Core.Message import *
from OCC.Core.TDF import *
from OCC.Core.BinObjMgt import *


class binmfunction:
	@staticmethod
	def AddDrivers(theDriverTable: BinMDF_ADriverTable, aMsgDrv: Message_Messenger) -> None: ...

class BinMFunction_FunctionDriver(BinMDF_ADriver):
	def __init__(self, theMessageDriver: Message_Messenger) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, Source: BinObjMgt_Persistent, Target: TDF_Attribute, RelocTable: BinObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, Source: TDF_Attribute, Target: BinObjMgt_Persistent, RelocTable: BinObjMgt_SRelocationTable) -> None: ...

class BinMFunction_GraphNodeDriver(BinMDF_ADriver):
	def __init__(self, theMessageDriver: Message_Messenger) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, Source: BinObjMgt_Persistent, Target: TDF_Attribute, RelocTable: BinObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, Source: TDF_Attribute, Target: BinObjMgt_Persistent, RelocTable: BinObjMgt_SRelocationTable) -> None: ...

class BinMFunction_ScopeDriver(BinMDF_ADriver):
	def __init__(self, theMessageDriver: Message_Messenger) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, Source: BinObjMgt_Persistent, Target: TDF_Attribute, RelocTable: BinObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, Source: TDF_Attribute, Target: BinObjMgt_Persistent, RelocTable: BinObjMgt_SRelocationTable) -> None: ...

# harray1 classes
# harray2 classes
# hsequence classes

binmfunction_AddDrivers = binmfunction.AddDrivers



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
TFunction module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_tfunction.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _TFunction
else:
    import _TFunction

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _TFunction.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _TFunction.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _TFunction.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _TFunction.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _TFunction.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _TFunction.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _TFunction.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _TFunction.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _TFunction.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _TFunction.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _TFunction.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _TFunction.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _TFunction.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _TFunction.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _TFunction.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _TFunction.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _TFunction.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _TFunction:
_TFunction.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _TFunction.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.TDF
import OCC.Core.TCollection
import OCC.Core.TColStd

from enum import IntEnum
from OCC.Core.Exception import *

TFunction_ES_WrongDefinition = _TFunction.TFunction_ES_WrongDefinition
TFunction_ES_NotExecuted = _TFunction.TFunction_ES_NotExecuted
TFunction_ES_Executing = _TFunction.TFunction_ES_Executing
TFunction_ES_Succeeded = _TFunction.TFunction_ES_Succeeded
TFunction_ES_Failed = _TFunction.TFunction_ES_Failed


class TFunction_ExecutionStatus(IntEnum):
	TFunction_ES_WrongDefinition = 0
	TFunction_ES_NotExecuted = 1
	TFunction_ES_Executing = 2
	TFunction_ES_Succeeded = 3
	TFunction_ES_Failed = 4
TFunction_ES_WrongDefinition = TFunction_ExecutionStatus.TFunction_ES_WrongDefinition
TFunction_ES_NotExecuted = TFunction_ExecutionStatus.TFunction_ES_NotExecuted
TFunction_ES_Executing = TFunction_ExecutionStatus.TFunction_ES_Executing
TFunction_ES_Succeeded = TFunction_ExecutionStatus.TFunction_ES_Succeeded
TFunction_ES_Failed = TFunction_ExecutionStatus.TFunction_ES_Failed


def Handle_TFunction_Driver_Create() -> "opencascade::handle< TFunction_Driver >":
    return _TFunction.Handle_TFunction_Driver_Create()

def Handle_TFunction_Driver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TFunction_Driver >":
    return _TFunction.Handle_TFunction_Driver_DownCast(t)

def Handle_TFunction_Driver_IsNull(t: "opencascade::handle< TFunction_Driver > const &") -> "bool":
    return _TFunction.Handle_TFunction_Driver_IsNull(t)

def Handle_TFunction_DriverTable_Create() -> "opencascade::handle< TFunction_DriverTable >":
    return _TFunction.Handle_TFunction_DriverTable_Create()

def Handle_TFunction_DriverTable_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TFunction_DriverTable >":
    return _TFunction.Handle_TFunction_DriverTable_DownCast(t)

def Handle_TFunction_DriverTable_IsNull(t: "opencascade::handle< TFunction_DriverTable > const &") -> "bool":
    return _TFunction.Handle_TFunction_DriverTable_IsNull(t)

def Handle_TFunction_Function_Create() -> "opencascade::handle< TFunction_Function >":
    return _TFunction.Handle_TFunction_Function_Create()

def Handle_TFunction_Function_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TFunction_Function >":
    return _TFunction.Handle_TFunction_Function_DownCast(t)

def Handle_TFunction_Function_IsNull(t: "opencascade::handle< TFunction_Function > const &") -> "bool":
    return _TFunction.Handle_TFunction_Function_IsNull(t)

def Handle_TFunction_GraphNode_Create() -> "opencascade::handle< TFunction_GraphNode >":
    return _TFunction.Handle_TFunction_GraphNode_Create()

def Handle_TFunction_GraphNode_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TFunction_GraphNode >":
    return _TFunction.Handle_TFunction_GraphNode_DownCast(t)

def Handle_TFunction_GraphNode_IsNull(t: "opencascade::handle< TFunction_GraphNode > const &") -> "bool":
    return _TFunction.Handle_TFunction_GraphNode_IsNull(t)

def Handle_TFunction_Logbook_Create() -> "opencascade::handle< TFunction_Logbook >":
    return _TFunction.Handle_TFunction_Logbook_Create()

def Handle_TFunction_Logbook_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TFunction_Logbook >":
    return _TFunction.Handle_TFunction_Logbook_DownCast(t)

def Handle_TFunction_Logbook_IsNull(t: "opencascade::handle< TFunction_Logbook > const &") -> "bool":
    return _TFunction.Handle_TFunction_Logbook_IsNull(t)

def Handle_TFunction_Scope_Create() -> "opencascade::handle< TFunction_Scope >":
    return _TFunction.Handle_TFunction_Scope_Create()

def Handle_TFunction_Scope_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TFunction_Scope >":
    return _TFunction.Handle_TFunction_Scope_DownCast(t)

def Handle_TFunction_Scope_IsNull(t: "opencascade::handle< TFunction_Scope > const &") -> "bool":
    return _TFunction.Handle_TFunction_Scope_IsNull(t)

def Handle_TFunction_HArray1OfDataMapOfGUIDDriver_Create() -> "opencascade::handle< TFunction_HArray1OfDataMapOfGUIDDriver >":
    return _TFunction.Handle_TFunction_HArray1OfDataMapOfGUIDDriver_Create()

def Handle_TFunction_HArray1OfDataMapOfGUIDDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TFunction_HArray1OfDataMapOfGUIDDriver >":
    return _TFunction.Handle_TFunction_HArray1OfDataMapOfGUIDDriver_DownCast(t)

def Handle_TFunction_HArray1OfDataMapOfGUIDDriver_IsNull(t: "opencascade::handle< TFunction_HArray1OfDataMapOfGUIDDriver > const &") -> "bool":
    return _TFunction.Handle_TFunction_HArray1OfDataMapOfGUIDDriver_IsNull(t)
class TFunction_Array1OfDataMapOfGUIDDriver(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_Array1< TFunction_DataMapOfGUIDDriver >::iterator":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_begin(self)

    def end(self) -> "NCollection_Array1< TFunction_DataMapOfGUIDDriver >::iterator":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_end(self)

    def cbegin(self) -> "NCollection_Array1< TFunction_DataMapOfGUIDDriver >::const_iterator":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_cbegin(self)

    def cend(self) -> "NCollection_Array1< TFunction_DataMapOfGUIDDriver >::const_iterator":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_cend(self)

    def __init__(self, *args):
        _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_swiginit(self, _TFunction.new_TFunction_Array1OfDataMapOfGUIDDriver(*args))

    def Init(self, theValue: "TFunction_DataMapOfGUIDDriver") -> "void":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Init(self, theValue)

    def Size(self) -> "Standard_Integer":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Size(self)

    def Length(self) -> "Standard_Integer":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Length(self)

    def IsEmpty(self) -> "Standard_Boolean":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_IsEmpty(self)

    def Lower(self) -> "Standard_Integer":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Lower(self)

    def Upper(self) -> "Standard_Integer":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Upper(self)

    def IsDeletable(self) -> "Standard_Boolean":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_IsDeletable(self)

    def IsAllocated(self) -> "Standard_Boolean":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_IsAllocated(self)

    def Assign(self, theOther: "TFunction_Array1OfDataMapOfGUIDDriver") -> "NCollection_Array1< TFunction_DataMapOfGUIDDriver > &":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Assign(self, theOther)

    def Move(self, theOther: "TFunction_Array1OfDataMapOfGUIDDriver") -> "NCollection_Array1< TFunction_DataMapOfGUIDDriver > &":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Move(self, theOther)

    def Set(self, *args) -> "NCollection_Array1< TFunction_DataMapOfGUIDDriver > &":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Set(self, *args)

    def First(self) -> "TFunction_DataMapOfGUIDDriver const &":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_First(self)

    def ChangeFirst(self) -> "TFunction_DataMapOfGUIDDriver &":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_ChangeFirst(self)

    def Last(self) -> "TFunction_DataMapOfGUIDDriver const &":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Last(self)

    def ChangeLast(self) -> "TFunction_DataMapOfGUIDDriver &":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_ChangeLast(self)

    def Value(self, theIndex: "Standard_Integer const") -> "TFunction_DataMapOfGUIDDriver const &":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Value(self, theIndex)

    def ChangeValue(self, theIndex: "Standard_Integer const") -> "TFunction_DataMapOfGUIDDriver &":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_ChangeValue(self, theIndex)

    def __call__(self, *args) -> "TFunction_DataMapOfGUIDDriver &":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver___call__(self, *args)

    def SetValue(self, theIndex: "Standard_Integer const", theItem: "TFunction_DataMapOfGUIDDriver") -> "void":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_SetValue(self, theIndex, theItem)

    def Resize(self, theLower: "Standard_Integer const", theUpper: "Standard_Integer const", theToCopyData: "Standard_Boolean const") -> "void":
        return _TFunction.TFunction_Array1OfDataMapOfGUIDDriver_Resize(self, theLower, theUpper, theToCopyData)
    __swig_destroy__ = _TFunction.delete_TFunction_Array1OfDataMapOfGUIDDriver

    def __getitem__(self, index):
        if index + self.Lower() > self.Upper():
            raise IndexError("index out of range")
        else:
            return self.Value(index + self.Lower())

    def __setitem__(self, index, value):
        if index + self.Lower() > self.Upper():
            raise IndexError("index out of range")
        else:
            self.SetValue(index + self.Lower(), value)

    def __len__(self):
        return self.Length()

    def __iter__(self):
        self.low = self.Lower()
        self.up = self.Upper()
        self.current = self.Lower() - 1
        return self

    def next(self):
        if self.current >= self.Upper():
            raise StopIteration
        else:
            self.current += 1
        return self.Value(self.current)

    __next__ = next


# Register TFunction_Array1OfDataMapOfGUIDDriver in _TFunction:
_TFunction.TFunction_Array1OfDataMapOfGUIDDriver_swigregister(TFunction_Array1OfDataMapOfGUIDDriver)

class TFunction_DataMapOfGUIDDriver(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_DataMap< Standard_GUID,opencascade::handle< TFunction_Driver >,Standard_GUID >::iterator":
        return _TFunction.TFunction_DataMapOfGUIDDriver_begin(self)

    def end(self) -> "NCollection_DataMap< Standard_GUID,opencascade::handle< TFunction_Driver >,Standard_GUID >::iterator":
        return _TFunction.TFunction_DataMapOfGUIDDriver_end(self)

    def cbegin(self) -> "NCollection_DataMap< Standard_GUID,opencascade::handle< TFunction_Driver >,Standard_GUID >::const_iterator":
        return _TFunction.TFunction_DataMapOfGUIDDriver_cbegin(self)

    def cend(self) -> "NCollection_DataMap< Standard_GUID,opencascade::handle< TFunction_Driver >,Standard_GUID >::const_iterator":
        return _TFunction.TFunction_DataMapOfGUIDDriver_cend(self)

    def __init__(self, *args):
        _TFunction.TFunction_DataMapOfGUIDDriver_swiginit(self, _TFunction.new_TFunction_DataMapOfGUIDDriver(*args))

    def Exchange(self, theOther: "TFunction_DataMapOfGUIDDriver") -> "void":
        return _TFunction.TFunction_DataMapOfGUIDDriver_Exchange(self, theOther)

    def Assign(self, theOther: "TFunction_DataMapOfGUIDDriver") -> "NCollection_DataMap< Standard_GUID,opencascade::handle< TFunction_Driver >,Standard_GUID > &":
        return _TFunction.TFunction_DataMapOfGUIDDriver_Assign(self, theOther)

    def Set(self, theOther: "TFunction_DataMapOfGUIDDriver") -> "NCollection_DataMap< Standard_GUID,opencascade::handle< TFunction_Driver >,Standard_GUID > &":
        return _TFunction.TFunction_DataMapOfGUIDDriver_Set(self, theOther)

    def ReSize(self, N: "Standard_Integer const") -> "void":
        return _TFunction.TFunction_DataMapOfGUIDDriver_ReSize(self, N)

    def Bind(self, theKey: "Standard_GUID", theItem: "opencascade::handle< TFunction_Driver > const &") -> "Standard_Boolean":
        return _TFunction.TFunction_DataMapOfGUIDDriver_Bind(self, theKey, theItem)

    def Bound(self, theKey: "Standard_GUID", theItem: "opencascade::handle< TFunction_Driver > const &") -> "opencascade::handle< TFunction_Driver > *":
        return _TFunction.TFunction_DataMapOfGUIDDriver_Bound(self, theKey, theItem)

    def IsBound(self, theKey: "Standard_GUID") -> "Standard_Boolean":
        return _TFunction.TFunction_DataMapOfGUIDDriver_IsBound(self, theKey)

    def UnBind(self, theKey: "Standard_GUID") -> "Standard_Boolean":
        return _TFunction.TFunction_DataMapOfGUIDDriver_UnBind(self, theKey)

    def Seek(self, theKey: "Standard_GUID") -> "opencascade::handle< TFunction_Driver > const *":
        return _TFunction.TFunction_DataMapOfGUIDDriver_Seek(self, theKey)

    def Find(self, *args) -> "Standard_Boolean":
        return _TFunction.TFunction_DataMapOfGUIDDriver_Find(self, *args)

    def ChangeSeek(self, theKey: "Standard_GUID") -> "opencascade::handle< TFunction_Driver > *":
        return _TFunction.TFunction_DataMapOfGUIDDriver_ChangeSeek(self, theKey)

    def ChangeFind(self, theKey: "Standard_GUID") -> "opencascade::handle< TFunction_Driver > &":
        return _TFunction.TFunction_DataMapOfGUIDDriver_ChangeFind(self, theKey)

    def __call__(self, *args) -> "opencascade::handle< TFunction_Driver > &":
        return _TFunction.TFunction_DataMapOfGUIDDriver___call__(self, *args)

    def Clear(self, *args) -> "void":
        return _TFunction.TFunction_DataMapOfGUIDDriver_Clear(self, *args)
    __swig_destroy__ = _TFunction.delete_TFunction_DataMapOfGUIDDriver

    def Size(self) -> "Standard_Integer":
        return _TFunction.TFunction_DataMapOfGUIDDriver_Size(self)

# Register TFunction_DataMapOfGUIDDriver in _TFunction:
_TFunction.TFunction_DataMapOfGUIDDriver_swigregister(TFunction_DataMapOfGUIDDriver)

class TFunction_DataMapOfLabelListOfLabel(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_DataMap< TDF_Label,NCollection_List< TDF_Label >,TDF_LabelMapHasher >::iterator":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_begin(self)

    def end(self) -> "NCollection_DataMap< TDF_Label,NCollection_List< TDF_Label >,TDF_LabelMapHasher >::iterator":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_end(self)

    def cbegin(self) -> "NCollection_DataMap< TDF_Label,NCollection_List< TDF_Label >,TDF_LabelMapHasher >::const_iterator":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_cbegin(self)

    def cend(self) -> "NCollection_DataMap< TDF_Label,NCollection_List< TDF_Label >,TDF_LabelMapHasher >::const_iterator":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_cend(self)

    def __init__(self, *args):
        _TFunction.TFunction_DataMapOfLabelListOfLabel_swiginit(self, _TFunction.new_TFunction_DataMapOfLabelListOfLabel(*args))

    def Exchange(self, theOther: "TFunction_DataMapOfLabelListOfLabel") -> "void":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_Exchange(self, theOther)

    def Assign(self, theOther: "TFunction_DataMapOfLabelListOfLabel") -> "NCollection_DataMap< TDF_Label,TDF_LabelList,TDF_LabelMapHasher > &":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_Assign(self, theOther)

    def Set(self, theOther: "TFunction_DataMapOfLabelListOfLabel") -> "NCollection_DataMap< TDF_Label,TDF_LabelList,TDF_LabelMapHasher > &":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_Set(self, theOther)

    def ReSize(self, N: "Standard_Integer const") -> "void":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_ReSize(self, N)

    def Bind(self, theKey: "TDF_Label", theItem: "TDF_LabelList") -> "Standard_Boolean":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_Bind(self, theKey, theItem)

    def Bound(self, theKey: "TDF_Label", theItem: "TDF_LabelList") -> "NCollection_List< TDF_Label > *":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_Bound(self, theKey, theItem)

    def IsBound(self, theKey: "TDF_Label") -> "Standard_Boolean":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_IsBound(self, theKey)

    def UnBind(self, theKey: "TDF_Label") -> "Standard_Boolean":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_UnBind(self, theKey)

    def Seek(self, theKey: "TDF_Label") -> "NCollection_List< TDF_Label > const *":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_Seek(self, theKey)

    def Find(self, *args) -> "Standard_Boolean":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_Find(self, *args)

    def ChangeSeek(self, theKey: "TDF_Label") -> "NCollection_List< TDF_Label > *":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_ChangeSeek(self, theKey)

    def ChangeFind(self, theKey: "TDF_Label") -> "NCollection_List< TDF_Label > &":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_ChangeFind(self, theKey)

    def __call__(self, *args) -> "NCollection_List< TDF_Label > &":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel___call__(self, *args)

    def Clear(self, *args) -> "void":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_Clear(self, *args)
    __swig_destroy__ = _TFunction.delete_TFunction_DataMapOfLabelListOfLabel

    def Size(self) -> "Standard_Integer":
        return _TFunction.TFunction_DataMapOfLabelListOfLabel_Size(self)

# Register TFunction_DataMapOfLabelListOfLabel in _TFunction:
_TFunction.TFunction_DataMapOfLabelListOfLabel_swigregister(TFunction_DataMapOfLabelListOfLabel)

class TFunction_DoubleMapOfIntegerLabel(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _TFunction.TFunction_DoubleMapOfIntegerLabel_swiginit(self, _TFunction.new_TFunction_DoubleMapOfIntegerLabel(*args))

    def Exchange(self, theOther: "TFunction_DoubleMapOfIntegerLabel") -> "void":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_Exchange(self, theOther)

    def Assign(self, theOther: "TFunction_DoubleMapOfIntegerLabel") -> "NCollection_DoubleMap< Standard_Integer,TDF_Label,TColStd_MapIntegerHasher,TDF_LabelMapHasher > &":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_Assign(self, theOther)

    def Set(self, theOther: "TFunction_DoubleMapOfIntegerLabel") -> "NCollection_DoubleMap< Standard_Integer,TDF_Label,TColStd_MapIntegerHasher,TDF_LabelMapHasher > &":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_Set(self, theOther)

    def ReSize(self, N: "Standard_Integer const") -> "void":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_ReSize(self, N)

    def Bind(self, theKey1: "int const &", theKey2: "TDF_Label") -> "void":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_Bind(self, theKey1, theKey2)

    def AreBound(self, theKey1: "int const &", theKey2: "TDF_Label") -> "Standard_Boolean":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_AreBound(self, theKey1, theKey2)

    def IsBound1(self, theKey1: "int const &") -> "Standard_Boolean":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_IsBound1(self, theKey1)

    def IsBound2(self, theKey2: "TDF_Label") -> "Standard_Boolean":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_IsBound2(self, theKey2)

    def UnBind1(self, theKey1: "int const &") -> "Standard_Boolean":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_UnBind1(self, theKey1)

    def UnBind2(self, theKey2: "TDF_Label") -> "Standard_Boolean":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_UnBind2(self, theKey2)

    def Find1(self, *args) -> "Standard_Boolean":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_Find1(self, *args)

    def Seek1(self, theKey1: "int const &") -> "TDF_Label const *":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_Seek1(self, theKey1)

    def Find2(self, *args) -> "Standard_Boolean":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_Find2(self, *args)

    def Seek2(self, theKey2: "TDF_Label") -> "int const *":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_Seek2(self, theKey2)

    def Clear(self, *args) -> "void":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_Clear(self, *args)
    __swig_destroy__ = _TFunction.delete_TFunction_DoubleMapOfIntegerLabel

    def Size(self) -> "Standard_Integer":
        return _TFunction.TFunction_DoubleMapOfIntegerLabel_Size(self)

# Register TFunction_DoubleMapOfIntegerLabel in _TFunction:
_TFunction.TFunction_DoubleMapOfIntegerLabel_swigregister(TFunction_DoubleMapOfIntegerLabel)

class TFunction_Driver(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def Arguments(self, *args) -> "void":
        r"""
        The method fills-in the list by labels, where the arguments of the function are located.

        Parameters
        ----------
        args: TDF_LabelList

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Driver_Arguments(self, *args)

    def Execute(self, *args) -> "Standard_Integer":
        r"""
        Executes the function in this function driver and puts the impacted labels in the logbook log. arguments & results of functions ================================.

        Parameters
        ----------
        log: TFunction_Logbook

        Returns
        -------
        int

        """
        return _TFunction.TFunction_Driver_Execute(self, *args)

    def Init(self, *args) -> "void":
        r"""
        Initializes the label l for this function prior to its execution.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Driver_Init(self, *args)

    def Label(self, *args) -> "TDF_Label":
        r"""
        Returns the label of the driver for this function.

        Returns
        -------
        TDF_Label

        """
        return _TFunction.TFunction_Driver_Label(self, *args)

    def MustExecute(self, *args) -> "Standard_Boolean":
        r"""
        Analyzes the labels in the logbook log. returns true if attributes have been modified. if the function label itself has been modified, the function must be executed.

        Parameters
        ----------
        log: TFunction_Logbook

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_Driver_MustExecute(self, *args)

    def Results(self, *args) -> "void":
        r"""
        The method fills-in the list by labels, where the results of the function are located.

        Parameters
        ----------
        res: TDF_LabelList

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Driver_Results(self, *args)

    def Validate(self, *args) -> "void":
        r"""
        Validates labels of a function in <log>. this function is the one initialized in this function driver. warning in regeneration mode, the solver must call this method even if the function is not executed. execution of function =====================.

        Parameters
        ----------
        log: TFunction_Logbook

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Driver_Validate(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_TFunction_Driver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TFunction.delete_TFunction_Driver

# Register TFunction_Driver in _TFunction:
_TFunction.TFunction_Driver_swigregister(TFunction_Driver)

class TFunction_DriverTable(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Default constructor.

        Returns
        -------
        None

        """
        _TFunction.TFunction_DriverTable_swiginit(self, _TFunction.new_TFunction_DriverTable(*args))

    def AddDriver(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if the driver has been added successfully to the driver table.

        Parameters
        ----------
        guid: Standard_GUID
        driver: TFunction_Driver
        thread: int,optional
        	default value is 0

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_DriverTable_AddDriver(self, *args)

    def Clear(self, *args) -> "void":
        r"""
        Removes all drivers. returns true if the driver has been removed successfully.

        Returns
        -------
        None

        """
        return _TFunction.TFunction_DriverTable_Clear(self, *args)

    def DumpToString(self) -> "std::string":
        r"""DumpToString(TFunction_DriverTable self) -> std::string"""
        return _TFunction.TFunction_DriverTable_DumpToString(self)

    def FindDriver(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if the driver was found.

        Parameters
        ----------
        guid: Standard_GUID
        driver: TFunction_Driver
        thread: int,optional
        	default value is 0

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_DriverTable_FindDriver(self, *args)

    @staticmethod
    def Get(*args) -> "opencascade::handle< TFunction_DriverTable >":
        r"""
        Returns the driver table. if a driver does not exist, creates it.

        Returns
        -------
        opencascade::handle<TFunction_DriverTable>

        """
        return _TFunction.TFunction_DriverTable_Get(*args)

    def HasDriver(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if the driver exists in the driver table.

        Parameters
        ----------
        guid: Standard_GUID
        thread: int,optional
        	default value is 0

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_DriverTable_HasDriver(self, *args)

    def RemoveDriver(self, *args) -> "Standard_Boolean":
        r"""
        Removes a driver with the given guid. returns true if the driver has been removed successfully.

        Parameters
        ----------
        guid: Standard_GUID
        thread: int,optional
        	default value is 0

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_DriverTable_RemoveDriver(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_TFunction_DriverTable_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TFunction.delete_TFunction_DriverTable

# Register TFunction_DriverTable in _TFunction:
_TFunction.TFunction_DriverTable_swigregister(TFunction_DriverTable)

def TFunction_DriverTable_Get(*args) -> "opencascade::handle< TFunction_DriverTable >":
    r"""
    Returns the driver table. if a driver does not exist, creates it.

    Returns
    -------
    opencascade::handle<TFunction_DriverTable>

    """
    return _TFunction.TFunction_DriverTable_Get(*args)

class TFunction_Function(OCC.Core.TDF.TDF_Attribute):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        _TFunction.TFunction_Function_swiginit(self, _TFunction.new_TFunction_Function(*args))

    def DumpToString(self) -> "std::string":
        r"""DumpToString(TFunction_Function self) -> std::string"""
        return _TFunction.TFunction_Function_DumpToString(self)

    def Failed(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if the execution failed.

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_Function_Failed(self, *args)

    def GetDriverGUID(self, *args) -> "Standard_GUID const &":
        r"""
        Returns the guid for this function's driver.

        Returns
        -------
        Standard_GUID

        """
        return _TFunction.TFunction_Function_GetDriverGUID(self, *args)

    def GetFailure(self, *args) -> "Standard_Integer":
        r"""
        Returns an index of failure if the execution of this function failed. if this integer value is 0, no failure has occurred. implementation of attribute methods: ===================================.

        Returns
        -------
        int

        """
        return _TFunction.TFunction_Function_GetFailure(self, *args)

    @staticmethod
    def GetID(*args) -> "Standard_GUID const &":
        r"""
        Returns the guid for functions. returns a function found on the label. instance methods: ================.

        Returns
        -------
        Standard_GUID

        """
        return _TFunction.TFunction_Function_GetID(*args)

    @staticmethod
    def Set(*args) -> "opencascade::handle< TFunction_Function >":
        r"""
        Static methods: ============== finds or creates a function attribute on the label <l>. returns the function attribute.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        opencascade::handle<TFunction_Function>
        Finds or creates a function attribute on the label <l>. sets a driver id to the function. returns the function attribute.

        Parameters
        ----------
        L: TDF_Label
        DriverID: Standard_GUID

        Returns
        -------
        opencascade::handle<TFunction_Function>

        """
        return _TFunction.TFunction_Function_Set(*args)

    def SetDriverGUID(self, *args) -> "void":
        r"""
        Sets the driver for this function as that indentified by the guid guid.

        Parameters
        ----------
        guid: Standard_GUID

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Function_SetDriverGUID(self, *args)

    def SetFailure(self, *args) -> "void":
        r"""
        Sets the failed index.

        Parameters
        ----------
        mode: int,optional
        	default value is 0

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Function_SetFailure(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_TFunction_Function_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TFunction.delete_TFunction_Function

# Register TFunction_Function in _TFunction:
_TFunction.TFunction_Function_swigregister(TFunction_Function)

def TFunction_Function_GetID(*args) -> "Standard_GUID const &":
    r"""
    Returns the guid for functions. returns a function found on the label. instance methods: ================.

    Returns
    -------
    Standard_GUID

    """
    return _TFunction.TFunction_Function_GetID(*args)

def TFunction_Function_Set(*args) -> "opencascade::handle< TFunction_Function >":
    r"""
    Static methods: ============== finds or creates a function attribute on the label <l>. returns the function attribute.

    Parameters
    ----------
    L: TDF_Label

    Returns
    -------
    opencascade::handle<TFunction_Function>
    Finds or creates a function attribute on the label <l>. sets a driver id to the function. returns the function attribute.

    Parameters
    ----------
    L: TDF_Label
    DriverID: Standard_GUID

    Returns
    -------
    opencascade::handle<TFunction_Function>

    """
    return _TFunction.TFunction_Function_Set(*args)

class TFunction_GraphNode(OCC.Core.TDF.TDF_Attribute):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        _TFunction.TFunction_GraphNode_swiginit(self, _TFunction.new_TFunction_GraphNode(*args))

    def AddNext(self, *args) -> "Standard_Boolean":
        r"""
        Defines a reference to the function as a next one.

        Parameters
        ----------
        funcID: int

        Returns
        -------
        bool
        Defines a reference to the function as a next one.

        Parameters
        ----------
        func: TDF_Label

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_GraphNode_AddNext(self, *args)

    def AddPrevious(self, *args) -> "Standard_Boolean":
        r"""
        Defines a reference to the function as a previous one.

        Parameters
        ----------
        funcID: int

        Returns
        -------
        bool
        Defines a reference to the function as a previous one.

        Parameters
        ----------
        func: TDF_Label

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_GraphNode_AddPrevious(self, *args)

    def DumpToString(self) -> "std::string":
        r"""DumpToString(TFunction_GraphNode self) -> std::string"""
        return _TFunction.TFunction_GraphNode_DumpToString(self)

    @staticmethod
    def GetID(*args) -> "Standard_GUID const &":
        r"""
        Returns the guid for graphnode attribute. instant methods =============== constructor (empty).

        Returns
        -------
        Standard_GUID

        """
        return _TFunction.TFunction_GraphNode_GetID(*args)

    def GetNext(self, *args) -> "TColStd_MapOfInteger const &":
        r"""
        Returns a map of next functions.

        Returns
        -------
        TColStd_MapOfInteger

        """
        return _TFunction.TFunction_GraphNode_GetNext(self, *args)

    def GetPrevious(self, *args) -> "TColStd_MapOfInteger const &":
        r"""
        Returns a map of previous functions.

        Returns
        -------
        TColStd_MapOfInteger

        """
        return _TFunction.TFunction_GraphNode_GetPrevious(self, *args)

    def GetStatus(self, *args) -> "TFunction_ExecutionStatus":
        r"""
        Returns the execution status of the function.

        Returns
        -------
        TFunction_ExecutionStatus

        """
        return _TFunction.TFunction_GraphNode_GetStatus(self, *args)

    def RemoveAllNext(self, *args) -> "void":
        r"""
        Clears a map of next functions.

        Returns
        -------
        None

        """
        return _TFunction.TFunction_GraphNode_RemoveAllNext(self, *args)

    def RemoveAllPrevious(self, *args) -> "void":
        r"""
        Clears a map of previous functions.

        Returns
        -------
        None

        """
        return _TFunction.TFunction_GraphNode_RemoveAllPrevious(self, *args)

    def RemoveNext(self, *args) -> "Standard_Boolean":
        r"""
        Removes a reference to the function as a next one.

        Parameters
        ----------
        funcID: int

        Returns
        -------
        bool
        Removes a reference to the function as a next one.

        Parameters
        ----------
        func: TDF_Label

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_GraphNode_RemoveNext(self, *args)

    def RemovePrevious(self, *args) -> "Standard_Boolean":
        r"""
        Removes a reference to the function as a previous one.

        Parameters
        ----------
        funcID: int

        Returns
        -------
        bool
        Removes a reference to the function as a previous one.

        Parameters
        ----------
        func: TDF_Label

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_GraphNode_RemovePrevious(self, *args)

    @staticmethod
    def Set(*args) -> "opencascade::handle< TFunction_GraphNode >":
        r"""
        Static methods ============== finds or creates a graph node attribute at the label <l>. returns the attribute.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        opencascade::handle<TFunction_GraphNode>

        """
        return _TFunction.TFunction_GraphNode_Set(*args)

    def SetStatus(self, *args) -> "void":
        r"""
        Defines an execution status for a function. implementation of attribute methods ===================================.

        Parameters
        ----------
        status: TFunction_ExecutionStatus

        Returns
        -------
        None

        """
        return _TFunction.TFunction_GraphNode_SetStatus(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_TFunction_GraphNode_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TFunction.delete_TFunction_GraphNode

# Register TFunction_GraphNode in _TFunction:
_TFunction.TFunction_GraphNode_swigregister(TFunction_GraphNode)

def TFunction_GraphNode_GetID(*args) -> "Standard_GUID const &":
    r"""
    Returns the guid for graphnode attribute. instant methods =============== constructor (empty).

    Returns
    -------
    Standard_GUID

    """
    return _TFunction.TFunction_GraphNode_GetID(*args)

def TFunction_GraphNode_Set(*args) -> "opencascade::handle< TFunction_GraphNode >":
    r"""
    Static methods ============== finds or creates a graph node attribute at the label <l>. returns the attribute.

    Parameters
    ----------
    L: TDF_Label

    Returns
    -------
    opencascade::handle<TFunction_GraphNode>

    """
    return _TFunction.TFunction_GraphNode_Set(*args)

class TFunction_IFunction(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None
        A constructor. initializes the interface by the label of function.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        None

        """
        _TFunction.TFunction_IFunction_swiginit(self, _TFunction.new_TFunction_IFunction(*args))

    def Arguments(self, *args) -> "void":
        r"""
        The method fills-in the list by labels, where the arguments of the function are located.

        Parameters
        ----------
        args: TDF_LabelList

        Returns
        -------
        None

        """
        return _TFunction.TFunction_IFunction_Arguments(self, *args)

    @staticmethod
    def DeleteFunction(*args) -> "Standard_Boolean":
        r"""
        Deletes a function attached to a label <l>. it deletes a tfunction_function attribute and a tfunction_graphnode. it deletes the functions from the scope of function of this document.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_IFunction_DeleteFunction(*args)

    def GetAllFunctions(self, *args) -> "TFunction_DoubleMapOfIntegerLabel const &":
        r"""
        Returns the scope of all functions.

        Returns
        -------
        TFunction_DoubleMapOfIntegerLabel

        """
        return _TFunction.TFunction_IFunction_GetAllFunctions(self, *args)

    def GetDriver(self, *args) -> "opencascade::handle< TFunction_Driver >":
        r"""
        Returns a driver of the function.

        Parameters
        ----------
        thread: int,optional
        	default value is 0

        Returns
        -------
        opencascade::handle<TFunction_Driver>

        """
        return _TFunction.TFunction_IFunction_GetDriver(self, *args)

    def GetGraphNode(self, *args) -> "opencascade::handle< TFunction_GraphNode >":
        r"""
        Returns a graph node of the function.

        Returns
        -------
        opencascade::handle<TFunction_GraphNode>

        """
        return _TFunction.TFunction_IFunction_GetGraphNode(self, *args)

    def GetLogbook(self, *args) -> "opencascade::handle< TFunction_Logbook >":
        r"""
        Returns the logbook - keeper of modifications.

        Returns
        -------
        opencascade::handle<TFunction_Logbook>

        """
        return _TFunction.TFunction_IFunction_GetLogbook(self, *args)

    def GetNext(self, *args) -> "void":
        r"""
        Returns a list of next functions.

        Parameters
        ----------
        prev: TDF_LabelList

        Returns
        -------
        None

        """
        return _TFunction.TFunction_IFunction_GetNext(self, *args)

    def GetPrevious(self, *args) -> "void":
        r"""
        Returns a list of previous functions.

        Parameters
        ----------
        prev: TDF_LabelList

        Returns
        -------
        None

        """
        return _TFunction.TFunction_IFunction_GetPrevious(self, *args)

    def GetStatus(self, *args) -> "TFunction_ExecutionStatus":
        r"""
        Returns the execution status of the function.

        Returns
        -------
        TFunction_ExecutionStatus

        """
        return _TFunction.TFunction_IFunction_GetStatus(self, *args)

    def Init(self, *args) -> "void":
        r"""
        Initializes the interface by the label of function.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        None

        """
        return _TFunction.TFunction_IFunction_Init(self, *args)

    def Label(self, *args) -> "TDF_Label const &":
        r"""
        Returns a label of the function.

        Returns
        -------
        TDF_Label

        """
        return _TFunction.TFunction_IFunction_Label(self, *args)

    @staticmethod
    def NewFunction(*args) -> "Standard_Boolean":
        r"""
        Sets a new function attached to a label <l> with <id>. it creates a new tfunction_function attribute initialized by the <id>, a new tfunction_graphnode with an empty list of dependencies and the status equal to tfunction_es_wrongdefinition. it registers the function in the scope of functions for this document.

        Parameters
        ----------
        L: TDF_Label
        ID: Standard_GUID

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_IFunction_NewFunction(*args)

    def Results(self, *args) -> "void":
        r"""
        The method fills-in the list by labels, where the results of the function are located.

        Parameters
        ----------
        res: TDF_LabelList

        Returns
        -------
        None

        """
        return _TFunction.TFunction_IFunction_Results(self, *args)

    def SetStatus(self, *args) -> "void":
        r"""
        Defines an execution status for a function.

        Parameters
        ----------
        status: TFunction_ExecutionStatus

        Returns
        -------
        None

        """
        return _TFunction.TFunction_IFunction_SetStatus(self, *args)

    def UpdateDependencies(self, *args) -> "Standard_Boolean":
        r"""
        Updates dependencies for all functions of the scope. it returns false in case of an error. an empty constructor.

        Parameters
        ----------
        Access: TDF_Label

        Returns
        -------
        bool
        Updates the dependencies of this function only.

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_IFunction_UpdateDependencies(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _TFunction.delete_TFunction_IFunction

# Register TFunction_IFunction in _TFunction:
_TFunction.TFunction_IFunction_swigregister(TFunction_IFunction)

def TFunction_IFunction_DeleteFunction(*args) -> "Standard_Boolean":
    r"""
    Deletes a function attached to a label <l>. it deletes a tfunction_function attribute and a tfunction_graphnode. it deletes the functions from the scope of function of this document.

    Parameters
    ----------
    L: TDF_Label

    Returns
    -------
    bool

    """
    return _TFunction.TFunction_IFunction_DeleteFunction(*args)

def TFunction_IFunction_NewFunction(*args) -> "Standard_Boolean":
    r"""
    Sets a new function attached to a label <l> with <id>. it creates a new tfunction_function attribute initialized by the <id>, a new tfunction_graphnode with an empty list of dependencies and the status equal to tfunction_es_wrongdefinition. it registers the function in the scope of functions for this document.

    Parameters
    ----------
    L: TDF_Label
    ID: Standard_GUID

    Returns
    -------
    bool

    """
    return _TFunction.TFunction_IFunction_NewFunction(*args)

class TFunction_Iterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        An empty constructor.

        Returns
        -------
        None
        A constructor. initializes the iterator.

        Parameters
        ----------
        Access: TDF_Label

        Returns
        -------
        None

        """
        _TFunction.TFunction_Iterator_swiginit(self, _TFunction.new_TFunction_Iterator(*args))

    def Current(self, *args) -> "TDF_LabelList const &":
        r"""
        Returns the current list of functions. if the iterator uses the execution status, the returned list contains only the functions with 'not executed' status.

        Returns
        -------
        TDF_LabelList

        """
        return _TFunction.TFunction_Iterator_Current(self, *args)

    def DumpToString(self) -> "std::string":
        r"""DumpToString(TFunction_Iterator self) -> std::string"""
        return _TFunction.TFunction_Iterator_DumpToString(self)

    def GetMaxNbThreads(self, *args) -> "Standard_Integer":
        r"""
        Analyses the graph of dependencies and returns maximum number of threads may be used to calculate the model.

        Returns
        -------
        int

        """
        return _TFunction.TFunction_Iterator_GetMaxNbThreads(self, *args)

    def GetStatus(self, *args) -> "TFunction_ExecutionStatus":
        r"""
        A help-function aimed to help the user to check the status of retrurned function. it calls tfunction_graphnode::getstatus() inside.

        Parameters
        ----------
        func: TDF_Label

        Returns
        -------
        TFunction_ExecutionStatus

        """
        return _TFunction.TFunction_Iterator_GetStatus(self, *args)

    def GetUsageOfExecutionStatus(self, *args) -> "Standard_Boolean":
        r"""
        Returns usage of execution status by the iterator.

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_Iterator_GetUsageOfExecutionStatus(self, *args)

    def Init(self, *args) -> "void":
        r"""
        Initializes the iterator.

        Parameters
        ----------
        Access: TDF_Label

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Iterator_Init(self, *args)

    def More(self, *args) -> "Standard_Boolean":
        r"""
        Returns false if the graph of functions is fully iterated.

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_Iterator_More(self, *args)

    def Next(self, *args) -> "void":
        r"""
        Switches the iterator to the next list of current functions.

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Iterator_Next(self, *args)

    def SetStatus(self, *args) -> "void":
        r"""
        A help-function aimed to help the user to change the execution status of a function. it calls tfunction_graphnode::setstatus() inside.

        Parameters
        ----------
        func: TDF_Label
        status: TFunction_ExecutionStatus

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Iterator_SetStatus(self, *args)

    def SetUsageOfExecutionStatus(self, *args) -> "void":
        r"""
        Defines the mode of iteration - usage or not of the execution status. if the iterator takes into account the execution status, the method ::current() returns only 'not executed' functions while their status is not changed. if the iterator ignores the execution status, the method ::current() returns the functions following their dependencies and ignoring the execution status.

        Parameters
        ----------
        usage: bool

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Iterator_SetUsageOfExecutionStatus(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _TFunction.delete_TFunction_Iterator

# Register TFunction_Iterator in _TFunction:
_TFunction.TFunction_Iterator_swigregister(TFunction_Iterator)

class TFunction_Logbook(OCC.Core.TDF.TDF_Attribute):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructor (empty).

        Returns
        -------
        None

        """
        _TFunction.TFunction_Logbook_swiginit(self, _TFunction.new_TFunction_Logbook(*args))

    def Clear(self, *args) -> "void":
        r"""
        Clears this logbook to its default, empty state.

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Logbook_Clear(self, *args)

    def Done(self, *args) -> "void":
        r"""
        Sets status of execution.

        Parameters
        ----------
        status: bool

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Logbook_Done(self, *args)

    def DumpToString(self) -> "std::string":
        r"""DumpToString(TFunction_Logbook self) -> std::string"""
        return _TFunction.TFunction_Logbook_DumpToString(self)

    @staticmethod
    def GetID(*args) -> "Standard_GUID const &":
        r"""
        Returns the guid for logbook attribute.

        Returns
        -------
        Standard_GUID

        """
        return _TFunction.TFunction_Logbook_GetID(*args)

    def GetImpacted(self, *args) -> "TDF_LabelMap const &":
        r"""
        Returns the map of impacted labels contained in this logbook.

        Returns
        -------
        TDF_LabelMap

        """
        return _TFunction.TFunction_Logbook_GetImpacted(self, *args)

    def GetTouched(self, *args) -> "TDF_LabelMap const &":
        r"""
        Returns the map of touched labels in this logbook. a touched label is the one modified by the end user.

        Returns
        -------
        TDF_LabelMap

        """
        return _TFunction.TFunction_Logbook_GetTouched(self, *args)

    def GetValid(self, *args) -> "void":
        r"""
        Returns the map of valid labels in this logbook.

        Returns
        -------
        TDF_LabelMap
        No available documentation.

        Parameters
        ----------
        Ls: TDF_LabelMap

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Logbook_GetValid(self, *args)

    def IsDone(self, *args) -> "Standard_Boolean":
        r"""
        Returns status of execution.

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_Logbook_IsDone(self, *args)

    def IsEmpty(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_Logbook_IsEmpty(self, *args)

    def IsModified(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if the label l is touched or impacted. this method is called by <tfunction_functiondriver::mustexecute>. if <withchildren> is set to true, the method checks all the sublabels of <l> too.

        Parameters
        ----------
        L: TDF_Label
        WithChildren: bool,optional
        	default value is Standard_False

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_Logbook_IsModified(self, *args)

    @staticmethod
    def Set(*args) -> "opencascade::handle< TFunction_Logbook >":
        r"""
        Finds or creates a tfunction_logbook attribute at the root label accessed by <access>. returns the attribute.

        Parameters
        ----------
        Access: TDF_Label

        Returns
        -------
        opencascade::handle<TFunction_Logbook>

        """
        return _TFunction.TFunction_Logbook_Set(*args)

    def SetImpacted(self, *args) -> "void":
        r"""
        Sets the label l as an impacted label in this logbook. this method is called by execution of the function driver.

        Parameters
        ----------
        L: TDF_Label
        WithChildren: bool,optional
        	default value is Standard_False

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Logbook_SetImpacted(self, *args)

    def SetTouched(self, *args) -> "void":
        r"""
        Sets the label l as a touched label in this logbook. in other words, l is understood to have been modified by the end user.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Logbook_SetTouched(self, *args)

    def SetValid(self, *args) -> "void":
        r"""
        Sets the label l as a valid label in this logbook.

        Parameters
        ----------
        L: TDF_Label
        WithChildren: bool,optional
        	default value is Standard_False

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        Ls: TDF_LabelMap

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Logbook_SetValid(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_TFunction_Logbook_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TFunction.delete_TFunction_Logbook

# Register TFunction_Logbook in _TFunction:
_TFunction.TFunction_Logbook_swigregister(TFunction_Logbook)

def TFunction_Logbook_GetID(*args) -> "Standard_GUID const &":
    r"""
    Returns the guid for logbook attribute.

    Returns
    -------
    Standard_GUID

    """
    return _TFunction.TFunction_Logbook_GetID(*args)

def TFunction_Logbook_Set(*args) -> "opencascade::handle< TFunction_Logbook >":
    r"""
    Finds or creates a tfunction_logbook attribute at the root label accessed by <access>. returns the attribute.

    Parameters
    ----------
    Access: TDF_Label

    Returns
    -------
    opencascade::handle<TFunction_Logbook>

    """
    return _TFunction.TFunction_Logbook_Set(*args)

class TFunction_Scope(OCC.Core.TDF.TDF_Attribute):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        _TFunction.TFunction_Scope_swiginit(self, _TFunction.new_TFunction_Scope(*args))

    def AddFunction(self, *args) -> "Standard_Boolean":
        r"""
        Adds a function to the scope of functions.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_Scope_AddFunction(self, *args)

    def ChangeFunctions(self, *args) -> "TFunction_DoubleMapOfIntegerLabel &":
        r"""
        Returns the scope of functions for modification. warning: don't use this method if you are not sure what you do!.

        Returns
        -------
        TFunction_DoubleMapOfIntegerLabel

        """
        return _TFunction.TFunction_Scope_ChangeFunctions(self, *args)

    def DumpToString(self) -> "std::string":
        r"""DumpToString(TFunction_Scope self) -> std::string"""
        return _TFunction.TFunction_Scope_DumpToString(self)

    def GetFreeID(self, *args) -> "Standard_Integer":
        r"""
        No available documentation.

        Returns
        -------
        int

        """
        return _TFunction.TFunction_Scope_GetFreeID(self, *args)

    def GetFunction(self, *args) -> "TDF_Label const &":
        r"""
        Returns an id of the function.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        int
        Returns the label of the function with this id.

        Parameters
        ----------
        ID: int

        Returns
        -------
        TDF_Label

        """
        return _TFunction.TFunction_Scope_GetFunction(self, *args)

    def GetFunctions(self, *args) -> "TFunction_DoubleMapOfIntegerLabel const &":
        r"""
        Returns the scope of functions.

        Returns
        -------
        TFunction_DoubleMapOfIntegerLabel

        """
        return _TFunction.TFunction_Scope_GetFunctions(self, *args)

    @staticmethod
    def GetID(*args) -> "Standard_GUID const &":
        r"""
        Returns the guid for scope attribute. instant methods =============== constructor (empty).

        Returns
        -------
        Standard_GUID

        """
        return _TFunction.TFunction_Scope_GetID(*args)

    def GetLogbook(self, *args) -> "opencascade::handle< TFunction_Logbook >":
        r"""
        Returns the logbook used in tfunction_driver methods. implementation of attribute methods ===================================.

        Returns
        -------
        opencascade::handle<TFunction_Logbook>

        """
        return _TFunction.TFunction_Scope_GetLogbook(self, *args)

    def HasFunction(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if the function exists with such an id.

        Parameters
        ----------
        ID: int

        Returns
        -------
        bool
        Returns true if the label contains a function of this scope.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_Scope_HasFunction(self, *args)

    def RemoveAllFunctions(self, *args) -> "void":
        r"""
        Removes all functions from the scope of functions.

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Scope_RemoveAllFunctions(self, *args)

    def RemoveFunction(self, *args) -> "Standard_Boolean":
        r"""
        Removes a function from the scope of functions.

        Parameters
        ----------
        L: TDF_Label

        Returns
        -------
        bool
        Removes a function from the scope of functions.

        Parameters
        ----------
        ID: int

        Returns
        -------
        bool

        """
        return _TFunction.TFunction_Scope_RemoveFunction(self, *args)

    @staticmethod
    def Set(*args) -> "opencascade::handle< TFunction_Scope >":
        r"""
        Static methods ============== finds or creates a tfunction_scope attribute at the root label accessed by <access>. returns the attribute.

        Parameters
        ----------
        Access: TDF_Label

        Returns
        -------
        opencascade::handle<TFunction_Scope>

        """
        return _TFunction.TFunction_Scope_Set(*args)

    def SetFreeID(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        ID: int

        Returns
        -------
        None

        """
        return _TFunction.TFunction_Scope_SetFreeID(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_TFunction_Scope_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TFunction.delete_TFunction_Scope

# Register TFunction_Scope in _TFunction:
_TFunction.TFunction_Scope_swigregister(TFunction_Scope)

def TFunction_Scope_GetID(*args) -> "Standard_GUID const &":
    r"""
    Returns the guid for scope attribute. instant methods =============== constructor (empty).

    Returns
    -------
    Standard_GUID

    """
    return _TFunction.TFunction_Scope_GetID(*args)

def TFunction_Scope_Set(*args) -> "opencascade::handle< TFunction_Scope >":
    r"""
    Static methods ============== finds or creates a tfunction_scope attribute at the root label accessed by <access>. returns the attribute.

    Parameters
    ----------
    Access: TDF_Label

    Returns
    -------
    opencascade::handle<TFunction_Scope>

    """
    return _TFunction.TFunction_Scope_Set(*args)

class TFunction_HArray1OfDataMapOfGUIDDriver(TFunction_Array1OfDataMapOfGUIDDriver, OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _TFunction.TFunction_HArray1OfDataMapOfGUIDDriver_swiginit(self, _TFunction.new_TFunction_HArray1OfDataMapOfGUIDDriver(*args))

    def Array1(self) -> "TFunction_Array1OfDataMapOfGUIDDriver const &":
        return _TFunction.TFunction_HArray1OfDataMapOfGUIDDriver_Array1(self)

    def ChangeArray1(self) -> "TFunction_Array1OfDataMapOfGUIDDriver &":
        return _TFunction.TFunction_HArray1OfDataMapOfGUIDDriver_ChangeArray1(self)


    @staticmethod
    def DownCast(t):
      return Handle_TFunction_HArray1OfDataMapOfGUIDDriver_DownCast(t)

    __swig_destroy__ = _TFunction.delete_TFunction_HArray1OfDataMapOfGUIDDriver

# Register TFunction_HArray1OfDataMapOfGUIDDriver in _TFunction:
_TFunction.TFunction_HArray1OfDataMapOfGUIDDriver_swigregister(TFunction_HArray1OfDataMapOfGUIDDriver)








# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
BRepAlgo module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_brepalgo.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _BRepAlgo
else:
    import _BRepAlgo

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _BRepAlgo.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _BRepAlgo.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _BRepAlgo.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _BRepAlgo.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _BRepAlgo.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _BRepAlgo.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _BRepAlgo.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _BRepAlgo.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _BRepAlgo.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _BRepAlgo.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _BRepAlgo.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _BRepAlgo.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _BRepAlgo.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _BRepAlgo.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _BRepAlgo.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _BRepAlgo.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _BRepAlgo.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _BRepAlgo:
_BRepAlgo.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _BRepAlgo.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.TopoDS
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.TopAbs
import OCC.Core.TopLoc
import OCC.Core.gp
import OCC.Core.GeomAbs
import OCC.Core.TopTools
import OCC.Core.BRepBuilderAPI
import OCC.Core.Geom
import OCC.Core.TColgp
import OCC.Core.BRepTools
import OCC.Core.Bnd
import OCC.Core.BVH
import OCC.Core.Geom2d
import OCC.Core.BRep
import OCC.Core.Poly
import OCC.Core.TShort
import OCC.Core.TopOpeBRepBuild
import OCC.Core.TopOpeBRepDS
import OCC.Core.TopOpeBRepTool
import OCC.Core.BRepClass3d
import OCC.Core.IntCurveSurface
import OCC.Core.math
import OCC.Core.Adaptor3d
import OCC.Core.Adaptor2d
import OCC.Core.Intf
import OCC.Core.IntSurf
import OCC.Core.IntCurvesFace
import OCC.Core.BRepAdaptor
import OCC.Core.GeomAdaptor
import OCC.Core.Geom2dAdaptor
import OCC.Core.TopExp
import OCC.Core.Extrema

from enum import IntEnum
from OCC.Core.Exception import *

BRepAlgo_OK = _BRepAlgo.BRepAlgo_OK
BRepAlgo_NOK = _BRepAlgo.BRepAlgo_NOK


class BRepAlgo_CheckStatus(IntEnum):
	BRepAlgo_OK = 0
	BRepAlgo_NOK = 1
BRepAlgo_OK = BRepAlgo_CheckStatus.BRepAlgo_OK
BRepAlgo_NOK = BRepAlgo_CheckStatus.BRepAlgo_NOK


def Handle_BRepAlgo_AsDes_Create() -> "opencascade::handle< BRepAlgo_AsDes >":
    return _BRepAlgo.Handle_BRepAlgo_AsDes_Create()

def Handle_BRepAlgo_AsDes_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< BRepAlgo_AsDes >":
    return _BRepAlgo.Handle_BRepAlgo_AsDes_DownCast(t)

def Handle_BRepAlgo_AsDes_IsNull(t: "opencascade::handle< BRepAlgo_AsDes > const &") -> "bool":
    return _BRepAlgo.Handle_BRepAlgo_AsDes_IsNull(t)
class brepalgo(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def ConcatenateWire(*args) -> "TopoDS_Wire":
        r"""
        This method makes a wire whose edges are c1 from a wire whose edges could be g1. it removes a vertex between g1 edges. option can be g1 or c1.

        Parameters
        ----------
        Wire: TopoDS_Wire
        Option: GeomAbs_Shape
        AngularTolerance: float,optional
        	default value is 1.0e-4

        Returns
        -------
        TopoDS_Wire

        """
        return _BRepAlgo.brepalgo_ConcatenateWire(*args)

    @staticmethod
    def ConcatenateWireC0(*args) -> "TopoDS_Edge":
        r"""
        This method makes an edge from a wire. junction points between edges of wire may be sharp, resulting curve of the resulting edge may be c0.

        Parameters
        ----------
        Wire: TopoDS_Wire

        Returns
        -------
        TopoDS_Edge

        """
        return _BRepAlgo.brepalgo_ConcatenateWireC0(*args)

    @staticmethod
    def IsTopologicallyValid(*args) -> "Standard_Boolean":
        r"""
        Checks if the shape is 'correct'. if not, returns <standard_false>, else returns <standard_true>. this method differs from the previous one in the fact that no geometric contols (intersection of wires, pcurve validity) are performed.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _BRepAlgo.brepalgo_IsTopologicallyValid(*args)

    @staticmethod
    def IsValid(*args) -> "Standard_Boolean":
        r"""
        Checks if the shape is 'correct'. if not, returns <standard_false>, else returns <standard_true>.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        bool
        Checks if the generated and modified faces from the shapes <arguments> in the shape <result> are 'correct'. the args may be empty, then all faces will be checked. if <closed> is true, only closed shape are valid. if <geomctrl> is false the geometry of new vertices and edges are not verified and the auto-intersection of new wires are not searched.

        Parameters
        ----------
        theArgs: TopTools_ListOfShape
        theResult: TopoDS_Shape
        closedSolid: bool,optional
        	default value is Standard_False
        GeomCtrl: bool,optional
        	default value is Standard_True

        Returns
        -------
        bool

        """
        return _BRepAlgo.brepalgo_IsValid(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _BRepAlgo.brepalgo_swiginit(self, _BRepAlgo.new_brepalgo())
    __swig_destroy__ = _BRepAlgo.delete_brepalgo

# Register brepalgo in _BRepAlgo:
_BRepAlgo.brepalgo_swigregister(brepalgo)

def brepalgo_ConcatenateWire(*args) -> "TopoDS_Wire":
    r"""
    This method makes a wire whose edges are c1 from a wire whose edges could be g1. it removes a vertex between g1 edges. option can be g1 or c1.

    Parameters
    ----------
    Wire: TopoDS_Wire
    Option: GeomAbs_Shape
    AngularTolerance: float,optional
    	default value is 1.0e-4

    Returns
    -------
    TopoDS_Wire

    """
    return _BRepAlgo.brepalgo_ConcatenateWire(*args)

def brepalgo_ConcatenateWireC0(*args) -> "TopoDS_Edge":
    r"""
    This method makes an edge from a wire. junction points between edges of wire may be sharp, resulting curve of the resulting edge may be c0.

    Parameters
    ----------
    Wire: TopoDS_Wire

    Returns
    -------
    TopoDS_Edge

    """
    return _BRepAlgo.brepalgo_ConcatenateWireC0(*args)

def brepalgo_IsTopologicallyValid(*args) -> "Standard_Boolean":
    r"""
    Checks if the shape is 'correct'. if not, returns <standard_false>, else returns <standard_true>. this method differs from the previous one in the fact that no geometric contols (intersection of wires, pcurve validity) are performed.

    Parameters
    ----------
    S: TopoDS_Shape

    Returns
    -------
    bool

    """
    return _BRepAlgo.brepalgo_IsTopologicallyValid(*args)

def brepalgo_IsValid(*args) -> "Standard_Boolean":
    r"""
    Checks if the shape is 'correct'. if not, returns <standard_false>, else returns <standard_true>.

    Parameters
    ----------
    S: TopoDS_Shape

    Returns
    -------
    bool
    Checks if the generated and modified faces from the shapes <arguments> in the shape <result> are 'correct'. the args may be empty, then all faces will be checked. if <closed> is true, only closed shape are valid. if <geomctrl> is false the geometry of new vertices and edges are not verified and the auto-intersection of new wires are not searched.

    Parameters
    ----------
    theArgs: TopTools_ListOfShape
    theResult: TopoDS_Shape
    closedSolid: bool,optional
    	default value is Standard_False
    GeomCtrl: bool,optional
    	default value is Standard_True

    Returns
    -------
    bool

    """
    return _BRepAlgo.brepalgo_IsValid(*args)

class BRepAlgo_AsDes(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty asdes.

        Returns
        -------
        None

        """
        _BRepAlgo.BRepAlgo_AsDes_swiginit(self, _BRepAlgo.new_BRepAlgo_AsDes(*args))

    def Add(self, *args) -> "void":
        r"""
        Stores <ss> as a futur subshape of <s>.

        Parameters
        ----------
        S: TopoDS_Shape
        SS: TopoDS_Shape

        Returns
        -------
        None
        Stores <ss> as futurs subshapes of <s>.

        Parameters
        ----------
        S: TopoDS_Shape
        SS: TopTools_ListOfShape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_AsDes_Add(self, *args)

    def Ascendant(self, *args) -> "TopTools_ListOfShape const &":
        r"""
        Returns the shape containing <s>.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopTools_ListOfShape

        """
        return _BRepAlgo.BRepAlgo_AsDes_Ascendant(self, *args)

    def ChangeDescendant(self, *args) -> "TopTools_ListOfShape &":
        r"""
        Returns futur subhapes of <s>.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopTools_ListOfShape

        """
        return _BRepAlgo.BRepAlgo_AsDes_ChangeDescendant(self, *args)

    def Clear(self, *args) -> "void":
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_AsDes_Clear(self, *args)

    def Descendant(self, *args) -> "TopTools_ListOfShape const &":
        r"""
        Returns futur subhapes of <s>.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopTools_ListOfShape

        """
        return _BRepAlgo.BRepAlgo_AsDes_Descendant(self, *args)

    def HasAscendant(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_AsDes_HasAscendant(self, *args)

    def HasCommonDescendant(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if (s1> and <s2> has common descendants. stores in <lc> the commons descendants.

        Parameters
        ----------
        S1: TopoDS_Shape
        S2: TopoDS_Shape
        LC: TopTools_ListOfShape

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_AsDes_HasCommonDescendant(self, *args)

    def HasDescendant(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_AsDes_HasDescendant(self, *args)

    def Remove(self, *args) -> "void":
        r"""
        Remove <s> from me.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_AsDes_Remove(self, *args)

    def Replace(self, *args) -> "void":
        r"""
        Replace <olds> by <news>. <olds> disapear from <self>.

        Parameters
        ----------
        OldS: TopoDS_Shape
        NewS: TopoDS_Shape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_AsDes_Replace(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_BRepAlgo_AsDes_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_AsDes

# Register BRepAlgo_AsDes in _BRepAlgo:
_BRepAlgo.BRepAlgo_AsDes_swigregister(BRepAlgo_AsDes)

class BRepAlgo_BooleanOperation(OCC.Core.BRepBuilderAPI.BRepBuilderAPI_MakeShape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def Builder(self, *args) -> "opencascade::handle< TopOpeBRepBuild_HBuilder >":
        r"""
        No available documentation.

        Returns
        -------
        opencascade::handle<TopOpeBRepBuild_HBuilder>

        """
        return _BRepAlgo.BRepAlgo_BooleanOperation_Builder(self, *args)

    def Perform(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        St1: TopAbs_State
        St2: TopAbs_State

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_BooleanOperation_Perform(self, *args)

    def PerformDS(self, *args) -> "void":
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_BooleanOperation_PerformDS(self, *args)

    def Shape1(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns the first shape involved in this boolean operation.

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepAlgo.BRepAlgo_BooleanOperation_Shape1(self, *args)

    def Shape2(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns the second shape involved in this boolean operation.

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepAlgo.BRepAlgo_BooleanOperation_Shape2(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_BooleanOperation

# Register BRepAlgo_BooleanOperation in _BRepAlgo:
_BRepAlgo.BRepAlgo_BooleanOperation_swigregister(BRepAlgo_BooleanOperation)

class BRepAlgo_FaceRestrictor(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        _BRepAlgo.BRepAlgo_FaceRestrictor_swiginit(self, _BRepAlgo.new_BRepAlgo_FaceRestrictor(*args))

    def Add(self, *args) -> "void":
        r"""
        Add the wire <w> to the set of wires. //! warning: the wires must be closed. //! the edges of <w> can be modified if they have not pcurves on the surface <s> of <f>. in this case if <proj> is false the first pcurve of the edge is positionned on <s>. if <proj> is true ,the pcurve on <s> is the projection of the curve 3d on <f>.

        Parameters
        ----------
        W: TopoDS_Wire

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_FaceRestrictor_Add(self, *args)

    def Clear(self, *args) -> "void":
        r"""
        Removes all the wires.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_FaceRestrictor_Clear(self, *args)

    def Current(self, *args) -> "TopoDS_Face":
        r"""
        No available documentation.

        Returns
        -------
        TopoDS_Face

        """
        return _BRepAlgo.BRepAlgo_FaceRestrictor_Current(self, *args)

    def Init(self, *args) -> "void":
        r"""
        The surface of <f> will be the the surface of each new faces built. <proj> is used to update pcurves on edges if necessary. see add().

        Parameters
        ----------
        F: TopoDS_Face
        Proj: bool,optional
        	default value is Standard_False
        ControlOrientation: bool,optional
        	default value is Standard_False

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_FaceRestrictor_Init(self, *args)

    def IsDone(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_FaceRestrictor_IsDone(self, *args)

    def More(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_FaceRestrictor_More(self, *args)

    def Next(self, *args) -> "void":
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_FaceRestrictor_Next(self, *args)

    def Perform(self, *args) -> "void":
        r"""
        Evaluate all the faces limited by the set of wires.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_FaceRestrictor_Perform(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_FaceRestrictor

# Register BRepAlgo_FaceRestrictor in _BRepAlgo:
_BRepAlgo.BRepAlgo_FaceRestrictor_swigregister(BRepAlgo_FaceRestrictor)

class BRepAlgo_Image(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        _BRepAlgo.BRepAlgo_Image_swiginit(self, _BRepAlgo.new_BRepAlgo_Image(*args))

    def Add(self, *args) -> "void":
        r"""
        Add <news> to the image of <olds>.

        Parameters
        ----------
        OldS: TopoDS_Shape
        NewS: TopoDS_Shape

        Returns
        -------
        None
        Add <news> to the image of <olds>.

        Parameters
        ----------
        OldS: TopoDS_Shape
        NewS: TopTools_ListOfShape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Image_Add(self, *args)

    def Bind(self, *args) -> "void":
        r"""
        Links <news> as image of <olds>.

        Parameters
        ----------
        OldS: TopoDS_Shape
        NewS: TopoDS_Shape

        Returns
        -------
        None
        Links <news> as image of <olds>.

        Parameters
        ----------
        OldS: TopoDS_Shape
        NewS: TopTools_ListOfShape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Image_Bind(self, *args)

    def Clear(self, *args) -> "void":
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Image_Clear(self, *args)

    def Compact(self, *args) -> "void":
        r"""
        Keeps only the link between roots and lastimage.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Image_Compact(self, *args)

    def Filter(self, *args) -> "void":
        r"""
        Deletes in the images the shape of type <shapetype> which are not in <s>. warning: compact() must be call before.

        Parameters
        ----------
        S: TopoDS_Shape
        ShapeType: TopAbs_ShapeEnum

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Image_Filter(self, *args)

    def HasImage(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_Image_HasImage(self, *args)

    def Image(self, *args) -> "TopTools_ListOfShape const &":
        r"""
        Returns the image of <s>. returns <s> in the list if hasimage(s) is false.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopTools_ListOfShape

        """
        return _BRepAlgo.BRepAlgo_Image_Image(self, *args)

    def ImageFrom(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns the generator of <s>.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepAlgo.BRepAlgo_Image_ImageFrom(self, *args)

    def IsImage(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_Image_IsImage(self, *args)

    def LastImage(self, *args) -> "void":
        r"""
        Stores in <l> the images of images of...images of <s>. <l> contains only <s> if hasimage(s) is false.

        Parameters
        ----------
        S: TopoDS_Shape
        L: TopTools_ListOfShape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Image_LastImage(self, *args)

    def Remove(self, *args) -> "void":
        r"""
        Remove <s> to set of images.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Image_Remove(self, *args)

    def RemoveRoot(self, *args) -> "void":
        r"""
        Removes the root <theroot> from the list of roots and up and down maps.

        Parameters
        ----------
        Root: TopoDS_Shape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Image_RemoveRoot(self, *args)

    def ReplaceRoot(self, *args) -> "void":
        r"""
        Replaces the <oldroot> with the <newroot>, so all images of the <oldroot> become the images of the <newroot>. the <oldroot> is removed.

        Parameters
        ----------
        OldRoot: TopoDS_Shape
        NewRoot: TopoDS_Shape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Image_ReplaceRoot(self, *args)

    def Root(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns the upper generator of <s>.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepAlgo.BRepAlgo_Image_Root(self, *args)

    def Roots(self, *args) -> "TopTools_ListOfShape const &":
        r"""
        No available documentation.

        Returns
        -------
        TopTools_ListOfShape

        """
        return _BRepAlgo.BRepAlgo_Image_Roots(self, *args)

    def SetRoot(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Image_SetRoot(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_Image

# Register BRepAlgo_Image in _BRepAlgo:
_BRepAlgo.BRepAlgo_Image_swigregister(BRepAlgo_Image)

class BRepAlgo_Loop(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        _BRepAlgo.BRepAlgo_Loop_swiginit(self, _BRepAlgo.new_BRepAlgo_Loop(*args))

    def AddConstEdge(self, *args) -> "void":
        r"""
        Add <e> as const edge, e can be in the result.

        Parameters
        ----------
        E: TopoDS_Edge

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Loop_AddConstEdge(self, *args)

    def AddConstEdges(self, *args) -> "void":
        r"""
        Add <le> as a set of const edges.

        Parameters
        ----------
        LE: TopTools_ListOfShape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Loop_AddConstEdges(self, *args)

    def AddEdge(self, *args) -> "void":
        r"""
        Add e with <lv>. <e> will be copied and trim by vertices in <lv>.

        Parameters
        ----------
        E: TopoDS_Edge
        LV: TopTools_ListOfShape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Loop_AddEdge(self, *args)

    def CutEdge(self, *args) -> "void":
        r"""
        Cut the edge <e> in several edges <ne> on the vertices<vone>.

        Parameters
        ----------
        E: TopoDS_Edge
        VonE: TopTools_ListOfShape
        NE: TopTools_ListOfShape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Loop_CutEdge(self, *args)

    def GetVerticesForSubstitute(self, *args) -> "void":
        r"""
        Returns the datamap of vertices with their substitutes.

        Parameters
        ----------
        VerVerMap: TopTools_DataMapOfShapeShape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Loop_GetVerticesForSubstitute(self, *args)

    def Init(self, *args) -> "void":
        r"""
        Init with <f> the set of edges must have pcurves on <f>.

        Parameters
        ----------
        F: TopoDS_Face

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Loop_Init(self, *args)

    def NewEdges(self, *args) -> "TopTools_ListOfShape const &":
        r"""
        Returns the list of new edges built from an edge <e> it can be an empty list.

        Parameters
        ----------
        E: TopoDS_Edge

        Returns
        -------
        TopTools_ListOfShape

        """
        return _BRepAlgo.BRepAlgo_Loop_NewEdges(self, *args)

    def NewFaces(self, *args) -> "TopTools_ListOfShape const &":
        r"""
        Returns the list of faces. warning: the method <wirestofaces> as to be called before. can be an empty list.

        Returns
        -------
        TopTools_ListOfShape

        """
        return _BRepAlgo.BRepAlgo_Loop_NewFaces(self, *args)

    def NewWires(self, *args) -> "TopTools_ListOfShape const &":
        r"""
        Returns the list of wires performed. can be an empty list.

        Returns
        -------
        TopTools_ListOfShape

        """
        return _BRepAlgo.BRepAlgo_Loop_NewWires(self, *args)

    def Perform(self, *args) -> "void":
        r"""
        Make loops.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Loop_Perform(self, *args)

    def VerticesForSubstitute(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        VerVerMap: TopTools_DataMapOfShapeShape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Loop_VerticesForSubstitute(self, *args)

    def WiresToFaces(self, *args) -> "void":
        r"""
        Build faces from the wires result.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Loop_WiresToFaces(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_Loop

# Register BRepAlgo_Loop in _BRepAlgo:
_BRepAlgo.BRepAlgo_Loop_swigregister(BRepAlgo_Loop)

class BRepAlgo_NormalProjection(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        None

        """
        _BRepAlgo.BRepAlgo_NormalProjection_swiginit(self, _BRepAlgo.new_BRepAlgo_NormalProjection(*args))

    def Add(self, *args) -> "void":
        r"""
        Add an edge or a wire to the list of shape to project.

        Parameters
        ----------
        ToProj: TopoDS_Shape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_Add(self, *args)

    def Ancestor(self, *args) -> "TopoDS_Shape const":
        r"""
        For a resulting edge, returns the corresponding initial edge.

        Parameters
        ----------
        E: TopoDS_Edge

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_Ancestor(self, *args)

    def Build(self, *args) -> "void":
        r"""
        Builds the result as a compound.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_Build(self, *args)

    def BuildWire(self, *args) -> "Standard_Boolean":
        r"""
        Build the result as a list of wire if possible in -- a first returns a wire only if there is only a wire.

        Parameters
        ----------
        Liste: TopTools_ListOfShape

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_BuildWire(self, *args)

    def Compute3d(self, *args) -> "void":
        r"""
        If with3d = standard_false the 3dcurve is not computed the initial 3dcurve is kept to build the resulting edges.

        Parameters
        ----------
        With3d: bool,optional
        	default value is Standard_True

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_Compute3d(self, *args)

    def Couple(self, *args) -> "TopoDS_Shape const":
        r"""
        For a projected edge, returns the corresponding initial face.

        Parameters
        ----------
        E: TopoDS_Edge

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_Couple(self, *args)

    def Generated(self, *args) -> "TopTools_ListOfShape const &":
        r"""
        Returns the list of shapes generated from the shape <s>.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopTools_ListOfShape

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_Generated(self, *args)

    def Init(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_Init(self, *args)

    def IsDone(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_IsDone(self, *args)

    def IsElementary(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        C: Adaptor3d_Curve

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_IsElementary(self, *args)

    def Projection(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns the result.

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_Projection(self, *args)

    def SetDefaultParams(self, *args) -> "void":
        r"""
        Set the parameters used for computation in their default values.

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_SetDefaultParams(self, *args)

    def SetLimit(self, *args) -> "void":
        r"""
        Manage limitation of projected edges.

        Parameters
        ----------
        FaceBoundaries: bool,optional
        	default value is Standard_True

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_SetLimit(self, *args)

    def SetMaxDistance(self, *args) -> "void":
        r"""
        Sets the maximum distance between target shape and shape to project. if this condition is not satisfied then corresponding part of solution is discarded. if maxdist < 0 then this method does not affect the algorithm.

        Parameters
        ----------
        MaxDist: float

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_SetMaxDistance(self, *args)

    def SetParams(self, *args) -> "void":
        r"""
        Set the parameters used for computation tol3d is the requiered tolerance between the 3d projected curve and its 2d representation internalcontinuity is the order of constraints used for approximation. maxdeg and maxseg are the maximum degree and the maximum number of segment for bspline resulting of an approximation.

        Parameters
        ----------
        Tol3D: float
        Tol2D: float
        InternalContinuity: GeomAbs_Shape
        MaxDegree: int
        MaxSeg: int

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_NormalProjection_SetParams(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_NormalProjection

# Register BRepAlgo_NormalProjection in _BRepAlgo:
_BRepAlgo.BRepAlgo_NormalProjection_swigregister(BRepAlgo_NormalProjection)

class BRepAlgo_Tool(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def Deboucle3D(*args) -> "TopoDS_Shape":
        r"""
        Remove the non valid part of an offsetshape 1 - remove all the free boundary and the faces connex to such edges. 2 - remove all the shapes not valid in the result (according to the side of offseting) in this verion only the first point is implemented.

        Parameters
        ----------
        S: TopoDS_Shape
        Boundary: TopTools_MapOfShape

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepAlgo.BRepAlgo_Tool_Deboucle3D(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _BRepAlgo.BRepAlgo_Tool_swiginit(self, _BRepAlgo.new_BRepAlgo_Tool())
    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_Tool

# Register BRepAlgo_Tool in _BRepAlgo:
_BRepAlgo.BRepAlgo_Tool_swigregister(BRepAlgo_Tool)

def BRepAlgo_Tool_Deboucle3D(*args) -> "TopoDS_Shape":
    r"""
    Remove the non valid part of an offsetshape 1 - remove all the free boundary and the faces connex to such edges. 2 - remove all the shapes not valid in the result (according to the side of offseting) in this verion only the first point is implemented.

    Parameters
    ----------
    S: TopoDS_Shape
    Boundary: TopTools_MapOfShape

    Returns
    -------
    TopoDS_Shape

    """
    return _BRepAlgo.BRepAlgo_Tool_Deboucle3D(*args)

class BRepAlgo_Common(BRepAlgo_BooleanOperation):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructs the common part of shapes s1 and s2.

        Parameters
        ----------
        S1: TopoDS_Shape
        S2: TopoDS_Shape

        Returns
        -------
        None

        """
        _BRepAlgo.BRepAlgo_Common_swiginit(self, _BRepAlgo.new_BRepAlgo_Common(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_Common

# Register BRepAlgo_Common in _BRepAlgo:
_BRepAlgo.BRepAlgo_Common_swigregister(BRepAlgo_Common)

class BRepAlgo_Cut(BRepAlgo_BooleanOperation):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Cuts the shape s2 from the shape s1.

        Parameters
        ----------
        S1: TopoDS_Shape
        S2: TopoDS_Shape

        Returns
        -------
        None

        """
        _BRepAlgo.BRepAlgo_Cut_swiginit(self, _BRepAlgo.new_BRepAlgo_Cut(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_Cut

# Register BRepAlgo_Cut in _BRepAlgo:
_BRepAlgo.BRepAlgo_Cut_swigregister(BRepAlgo_Cut)

class BRepAlgo_Fuse(BRepAlgo_BooleanOperation):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Fuse s1 and s2.

        Parameters
        ----------
        S1: TopoDS_Shape
        S2: TopoDS_Shape

        Returns
        -------
        None

        """
        _BRepAlgo.BRepAlgo_Fuse_swiginit(self, _BRepAlgo.new_BRepAlgo_Fuse(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_Fuse

# Register BRepAlgo_Fuse in _BRepAlgo:
_BRepAlgo.BRepAlgo_Fuse_swigregister(BRepAlgo_Fuse)

class BRepAlgo_Section(BRepAlgo_BooleanOperation):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        Sh1: TopoDS_Shape
        Sh2: TopoDS_Shape
        PerformNow: bool,optional
        	default value is Standard_True

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        Sh: TopoDS_Shape
        Pl: gp_Pln
        PerformNow: bool,optional
        	default value is Standard_True

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        Sh: TopoDS_Shape
        Sf: Geom_Surface
        PerformNow: bool,optional
        	default value is Standard_True

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        Sf: Geom_Surface
        Sh: TopoDS_Shape
        PerformNow: bool,optional
        	default value is Standard_True

        Returns
        -------
        None
        This and the above algorithms construct a framework for computing the section lines of - the two shapes sh1 and sh2, or - the shape sh and the plane pl, or - the shape sh and the surface sf, or - the surface sf and the shape sh, or - the two surfaces sf1 and sf2, and builds the result if performnow equals true, its default value. if performnow equals false, the intersection will be computed later by the function build. the constructed shape will be returned by the function shape. this is a compound object composed of edges. these intersection edges may be built: - on new intersection lines, or - on coincident portions of edges in the two intersected shapes. these intersection edges are independent: they are not chained or grouped in wires. if no intersection edge exists, the result is an empty compound object. note that other objects than topods_shape shapes involved in these syntaxes are converted into faces or shells before performing the computation of the intersection. a shape resulting from this conversion can be retrieved with the function shape1 or shape2. parametric 2d curves on intersection edges no parametric 2d curve (pcurve) is defined for each elementary edge of the result. to attach such parametric curves to the constructed edges you may use a constructor with the performnow flag equal to false; then you use: - the function computepcurveon1 to ask for the additional computation of a pcurve in the parametric space of the first shape, - the function computepcurveon2 to ask for the additional computation of a pcurve in the parametric space of the second shape, - in the end, the function build to construct the result. note that as a result, pcurves will only be added on edges built on new intersection lines. approximation of intersection edges the underlying 3d geometry attached to each elementary edge of the result is: - analytic where possible, provided the corresponding geometry corresponds to a type of analytic curve defined in the geom package; for example, the intersection of a cylindrical shape with a plane gives an ellipse or a circle; - or elsewhere, given as a succession of points grouped together in a bspline curve of degree 1. if you prefer to have an attached 3d geometry which is a bspline approximation of the computed set of points on computed elementary intersection edges whose underlying geometry is not analytic, you may use a constructor with the performnow flag equal to false. then you use: - the function approximation to ask for this computation option, and - the function build to construct the result. note that as a result, approximations will only be computed on edges built on new intersection lines. example you may also combine these computation options. in the following example: - each elementary edge of the computed intersection, built on a new intersection line, which does not correspond to an analytic geom curve, will be approximated by a bspline curve whose degree is not greater than 8. - each elementary edge built on a new intersection line, will have: - a pcurve in the parametric space of the shape s1, - no pcurve in the parametric space of the shape s2. // topods_shape s1 = ... , s2 = ... ; standard_boolean performnow = standard_false; brepalgo_section s ( s1, s2, performnow ); s.computepcurveon1 (standard_true); s.approximation (standard_true); s.build(); topods_shape r = s.shape();.

        Parameters
        ----------
        Sf1: Geom_Surface
        Sf2: Geom_Surface
        PerformNow: bool,optional
        	default value is Standard_True

        Returns
        -------
        None

        """
        _BRepAlgo.BRepAlgo_Section_swiginit(self, _BRepAlgo.new_BRepAlgo_Section(*args))

    def Approximation(self, *args) -> "void":
        r"""
        Defines an option for computation of further intersections. this computation will be performed by the function build in this framework. by default, the underlying 3d geometry attached to each elementary edge of the result of a computed intersection is: - analytic where possible, provided the corresponding geometry corresponds to a type of analytic curve defined in the geom package; for example the intersection of a cylindrical shape with a plane gives an ellipse or a circle; - or elsewhere, given as a succession of points grouped together in a bspline curve of degree 1. if approx equals true, when further computations are performed in this framework with the function build, these edges will have an attached 3d geometry which is a bspline approximation of the computed set of points. note that as a result, approximations will be computed on edges built only on new intersection lines.

        Parameters
        ----------
        B: bool

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Section_Approximation(self, *args)

    def ComputePCurveOn1(self, *args) -> "void":
        r"""
        Indicates if the pcurve must be (or not) performed on first part.

        Parameters
        ----------
        B: bool

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Section_ComputePCurveOn1(self, *args)

    def ComputePCurveOn2(self, *args) -> "void":
        r"""
        Define options for the computation of further intersections which will be performed by the function build in this framework. by default, no parametric 2d curve (pcurve) is defined for the elementary edges of the result. if computepcurve1 equals true, further computations performed in this framework with the function build will attach an additional pcurve in the parametric space of the first shape to the constructed edges. if computepcurve2 equals true, the additional pcurve will be attached to the constructed edges in the parametric space of the second shape. these two functions may be used together. note that as a result, pcurves will only be added onto edges built on new intersection lines.

        Parameters
        ----------
        B: bool

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Section_ComputePCurveOn2(self, *args)

    def HasAncestorFaceOn1(self, *args) -> "Standard_Boolean":
        r"""
        Identifies the ancestor faces of the new intersection edge e resulting from the last computation performed in this framework, that is, the faces of the two original shapes on which the edge e lies: - hasancestorfaceon1 gives the ancestor face in the first shape, and these functions return: - true if an ancestor face f is found, or - false if not. an ancestor face is identifiable for the edge e if the three following conditions are satisfied: - the first part on which this algorithm performed its last computation is a shape, that is, it was not given as a surface or a plane at the time of construction of this algorithm or at a later time by the init1 function, - e is one of the elementary edges built by the last computation of this section algorithm, - the edge e is built on an intersection curve. in other words, e is a new edge built on the intersection curve, not on edges belonging to the intersecting shapes. to use these functions properly, you have to test the returned boolean value before using the ancestor face: f is significant only if the returned boolean value equals true.

        Parameters
        ----------
        E: TopoDS_Shape
        F: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_Section_HasAncestorFaceOn1(self, *args)

    def HasAncestorFaceOn2(self, *args) -> "Standard_Boolean":
        r"""
        Identifies the ancestor faces of the new intersection edge e resulting from the last computation performed in this framework, that is, the faces of the two original shapes on which the edge e lies: - hasancestorfaceon2 gives the ancestor face in the second shape. these functions return: - true if an ancestor face f is found, or - false if not. an ancestor face is identifiable for the edge e if the three following conditions are satisfied: - the first part on which this algorithm performed its last computation is a shape, that is, it was not given as a surface or a plane at the time of construction of this algorithm or at a later time by the init1 function, - e is one of the elementary edges built by the last computation of this section algorithm, - the edge e is built on an intersection curve. in other words, e is a new edge built on the intersection curve, not on edges belonging to the intersecting shapes. to use these functions properly, you have to test the returned boolean value before using the ancestor face: f is significant only if the returned boolean value equals true.

        Parameters
        ----------
        E: TopoDS_Shape
        F: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _BRepAlgo.BRepAlgo_Section_HasAncestorFaceOn2(self, *args)

    def Init1(self, *args) -> "void":
        r"""
        Initializes the first part.

        Parameters
        ----------
        S1: TopoDS_Shape

        Returns
        -------
        None
        Initializes the first part.

        Parameters
        ----------
        Pl: gp_Pln

        Returns
        -------
        None
        Initializes the first part.

        Parameters
        ----------
        Sf: Geom_Surface

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Section_Init1(self, *args)

    def Init2(self, *args) -> "void":
        r"""
        Initialize second part.

        Parameters
        ----------
        S2: TopoDS_Shape

        Returns
        -------
        None
        Initializes the second part.

        Parameters
        ----------
        Pl: gp_Pln

        Returns
        -------
        None
        This and the above algorithms reinitialize the first and the second parts on which this algorithm is going to perform the intersection computation. this is done with either: the surface sf, the plane pl or the shape sh. you use the function build to construct the result.

        Parameters
        ----------
        Sf: Geom_Surface

        Returns
        -------
        None

        """
        return _BRepAlgo.BRepAlgo_Section_Init2(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepAlgo.delete_BRepAlgo_Section

# Register BRepAlgo_Section in _BRepAlgo:
_BRepAlgo.BRepAlgo_Section_swigregister(BRepAlgo_Section)





